/**
* Copyright (c) 2005, Freescale Semiconductor
* Freescale Willy Note
*
* File name : main.c
* Project name: GB60_IIC.mcp
*
* Author : Miguel Agnessi Melendez
*          Gonzalo Delgado Huitron
* Department : RTAC Americas
*
* Description : The example shown consists in a generic code to use the 
*               HCS08 MCU IIC module to communicate with another IIC 
*               device, using the HCS08 MCU IIC interrupt routine to 
*               handle most of the communication. This example is 
*               intended to be used in a basic scenario where the 
*               communication just addresses the slave and in the next 
*               byte starts sending or receiving data.This project was 
*               made with the Device Initialization tool which sets the 
*               module, according to the user's needs.
*                            
* (Refer to AN3136 for more information about Device Initialization)
*
* History :   
* 10/08/2005  : Release (rgd04c)
*/
#include <hidef.h> /* for EnableInterrupts macro */
#include "derivative.h" /* include peripheral declarations */
#include "MCUinit.h"

//#define MASTER

#define IIC_ERROR_STATUS 0
#define IIC_READY_STATUS 1
#define IIC_HEADER_SENT_STATUS 2
#define IIC_DATA_TRANSMISION_STATUS 3
#define IIC_DATA_SENT_STATUS 4

void configureI2C(unsigned char);
unsigned char WriteBytesI2C (unsigned char,unsigned char);
unsigned char ReadBytesI2C (unsigned char,unsigned char);

//----I2C Variables -------------------------------------------------------
unsigned char I2C_STEP= IIC_READY_STATUS;
unsigned char I2C_DATA_DIRECTION = 0;      /* 1 Transmit, 0 Read */
unsigned char I2C_LENGTH = 1;
unsigned char I2C_COUNTER = 0;

unsigned char I2C_DATA[128];			/* IIC Buffer */

//-------------------------------------------------------------------------


/* Main procedure. */
void main(void) {
 MCU_init(); 
  /* Configure internal clock reference should be done out of the DI
   * Internal clock and 19,995,428 bus frequency. */
  ICGC1 = 0x28;
  ICGC2 = 0x70;
  
  /* Configure interfaces. Set our IIC adress. */
#ifdef MASTER  
  configureI2C(0x50);
  I2C_DATA[0]='A';	/* test data */
#else
  configureI2C(0x52);
#endif 
     
  EnableInterrupts; /* enable interrupts */  
   
#ifdef MASTER  
 // ReadBytesI2C(0x52,6);

  WriteBytesI2C(0x52,1);
  

  while(I2C_STEP>IIC_READY_STATUS)__RESET_WATCHDOG(); /* wait for memory to be read */
#endif 
    
  /* Application is based on interrupts so just stay here forever. */
  for(;;) {          
      __RESET_WATCHDOG(); /* feeds the dog */
  }                       /* loop forever */
  /* please make sure that you never leave this function */
}



/* Function to configure the IIC module. */
void configureI2C(unsigned char selfAddress){
    IIC1C_IICEN = 1;      /* Enable IIC            */
    IIC1A = selfAddress;   /* IIC Address           */
    IIC1F = 0x8D;				   /* Set IIC frequency     */    
    I2C_STEP = IIC_READY_STATUS;
    IIC1C_IICIE = 1;      /* Enable IIC interrupts */
    
}



unsigned char WriteBytesI2C (unsigned char slaveAddress,unsigned char numberOfBytes){
	
	unsigned char Temp;
  
  I2C_LENGTH = numberOfBytes;
  I2C_COUNTER =0;
  I2C_STEP = IIC_HEADER_SENT_STATUS;
  I2C_DATA_DIRECTION = 1;
  
  /* Format the Address to fit in the IICA register and place a 0 on the R/W bit.*/
  slaveAddress &= 0xFE;

  IIC1C_IICEN = 0;
  IIC1C_IICEN = 1;
  Temp = IIC1S;                 /* Clear any pending interrupt */
  IIC1S_IICIF=1;

  IIC1C_MST = 0;
  
  IIC1S_SRW=0;
  IIC1C_TX = 1;				          /* Select Transmit Mode */
	IIC1C_MST = 1;				        /* Select Master Mode (Send Start Bit) */
    
  for(Temp=0;Temp<5;Temp++);    /* Small delay */
  
  IIC1D=slaveAddress;						/* Send selected slave address */
  
  return(1);
}


unsigned char ReadBytesI2C (unsigned char slaveAddress,unsigned char numberOfBytes){

  unsigned char Temp;

  I2C_LENGTH = numberOfBytes;

  I2C_COUNTER =0;
  I2C_STEP = IIC_HEADER_SENT_STATUS;
  I2C_DATA_DIRECTION = 0;
  
  /* Format the Address to fit in the IICA register and place a 1 on the R/W bit. */
 
  slaveAddress &= 0xFE;
  slaveAddress |= 0x01;         /* Set the Read from slave bit. */

  Temp = IIC1S;                 /* Clear any pending interrupt  */
  IIC1S_IICIF=1;
  
  IIC1C_TX = 1;				          /* Select Transmit Mode          */
	IIC1C_MST = 1;				        /* Select Master Mode (Send Start Bit)*/
  
  IIC1D=slaveAddress;						/* Send selected slave address   */
  
  return(1);
}


//-------------------------------------------------------------
#pragma TRAP_PROC


/* Interrupt handler routine to manage all the events related
 * to the IIC module.    */
interrupt 24 void IIC_Control_handler(void){

	unsigned char Temp;
		
	Temp = IIC1S;              /* ACK the interrupt */
  IIC1S_IICIF=1;
	  
  if(IIC1S_ARBL==1){	       /* Verify the Arbitration lost status */	     
	     IIC1S_ARBL= 1;
	     IIC1C_MST = 0;	
	     I2C_STEP = IIC_ERROR_STATUS;
	     return;     
	}										       /* If Arbitration is OK continue */  
		
	
	
	if(IIC1C_MST==1){          /* If we are the IIC Master */
	 				 	 
	 	 if(IIC1S_RXAK==1){      /* Verify if byte sent was ACK */
	 	    IIC1C_MST = 0;
	      I2C_STEP = IIC_ERROR_STATUS;
	      return;
	 	 }
	 			             
     if(I2C_STEP == IIC_HEADER_SENT_STATUS){ /* Header Sent */
          IIC1C_TX = I2C_DATA_DIRECTION;
          
          I2C_STEP = IIC_DATA_TRANSMISION_STATUS; 
          
          if(IIC1C_TX==0){  /* If we are reading data clock in first slave byte */
              Temp = IIC1D;
              return;
          }
     }
              
     if(I2C_STEP == IIC_DATA_TRANSMISION_STATUS){	 /* If byte transmision is in progress.*/
           
        	  if(IIC1C_TX==1){				               /* If Master is sending data to slave */     	 
        	            	    
        	    IIC1D = I2C_DATA[I2C_COUNTER];	     /* Send the next byte */
        	         	 
        	    I2C_COUNTER++;
        	    if(I2C_LENGTH <= I2C_COUNTER){        	     
        	      I2C_STEP=IIC_DATA_SENT_STATUS;     /* Mark we are done sending Bytes */   	  
        	    }
        	    return;        	 								     /* wait until last byte sent      */
        	  } 
        	  else{										               /* If master is reading data from slave */
        	      
        	      if((I2C_COUNTER+1) == I2C_LENGTH)  /* Master should not ACK the last byte */
        	          IIC1C_TXAK = 1;							   /* to indicate end of transfer         */
        	      
        	      I2C_DATA[I2C_COUNTER] = IIC1D;	   /* Read the next byte */       	         	 
        	      I2C_COUNTER++;
        	      																 
        	      if(I2C_LENGTH <= I2C_COUNTER){        	     
        	        I2C_STEP=IIC_DATA_SENT_STATUS;   /* Mark we are done sending Bytes */   	  
        	      }   
        	      
        	      return;           	    					 /* Return until next byte is read */
        	 }	  
       }
                   
       if(I2C_STEP==IIC_DATA_SENT_STATUS){	       /* We are done with the transmition.*/ 
        	 
        	 I2C_STEP=IIC_READY_STATUS;	             /* Reset our status flag            */
        	 Temp = IIC1S;					  				       /* ACK the interrupt                */
           IIC1S_IICIF=1;
        	 
        	 IIC1C_TX=0;
        	 IIC1S_SRW=0;
        	 IIC1C_MST=0;
        	 			                                   /* Generate a stop condition        */        	  
        	 return;
       }        
	}
  else{			 /*  SLAVE OPERATION  */  
                   																						 
       
       if(I2C_STEP <= IIC_READY_STATUS){	 				 /* If it is the first byte tranmited */
           I2C_STEP = IIC_DATA_TRANSMISION_STATUS;
           IIC1C_TX = IIC1S_SRW;				           /* Set the transmision reception status */
           I2C_COUNTER = 0;
           
           /* If we are receiving data read IIC1D to get free bus and get the next byte */
           if(IIC1C_TX==0){
              Temp = IIC1D;
              return;
           }
       }
       	
       
			 if(IIC1S_TCF==1){
          if(IIC1C_TX == 0){	        /* If data is received store it on the buffer */              
              I2C_DATA[I2C_COUNTER]=IIC1D;
              I2C_COUNTER++;
              return;          
          } 
          else{	                      /* Data sent by the slave */              
              
              if(IIC1S_RXAK==1){      /* If byte is not ACK end transmision. */
	                IIC1C_TX = 0;
	                Temp = IIC1D;
	                I2C_STEP = IIC_READY_STATUS;
	                return;
	 	          }
              
              IIC1D = I2C_DATA[I2C_COUNTER];
              I2C_COUNTER++;
              return;          
          }
		  }
   }
}